@@ -1,8 +1,5 @@
 """
-    tiamatpip.cli
-    ~~~~~~~~~~~~~
-
-    PIP handling for tiamat packaged python projects
+PIP handling for tiamat packaged python projects.
 """
 import code
 import logging
@@ -10,8 +7,11 @@ import os
 import pathlib
 import sys
 from typing import List
+from typing import Optional
 from typing import Sequence
+from typing import Union
 
+import pip._vendor.distlib
 from pip._internal.cli.main import main as pip_main
 from pip._internal.commands.install import InstallCommand
 from pip._internal.commands.uninstall import UninstallCommand
@@ -20,10 +20,20 @@ from tiamatpip import configure
 from tiamatpip.store import DistributionNotFound
 from tiamatpip.store import Store
 from tiamatpip.utils import debug_print
-from tiamatpip.utils import patch_pip_get_installed_distributions
+from tiamatpip.utils import patch_pip_internal_metadata_get_default_environment
 from tiamatpip.utils import patched_environ
 from tiamatpip.utils import patched_sys_argv
 
+
+if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
+    # We're running from a compiled pyinstaller binary and distlib knows
+    # nothing about this. It also does not know how to handle pyimod03_importers, so,
+    # we need to patch the __loader__ attribute to force distlib to it's fallback code.
+    # This is particularly important on Windows where failing to do so makes tiamat-pip
+    # unusable.
+    pip._vendor.distlib.__loader__ = None
+
+
 # If there are logging handlers already configured, then the basicConfig
 # call below will be a no-op
 logging.basicConfig(
@@ -34,7 +44,10 @@ logging.basicConfig(
 log = logging.getLogger(__name__)
 
 
-def should_redirect_argv(argv):
+def should_redirect_argv(argv: List[str]) -> bool:
+    """
+    Check if ``argv`` should be redirected.
+    """
     debug_print("should_redirect_argv", argv)
     if "TIAMAT_PIP_INSTALL" in os.environ:
         # A pip command is already in progress. This is usually
@@ -47,8 +60,12 @@ def should_redirect_argv(argv):
     return False
 
 
-def process_argv(argv: List[str]) -> bool:
-    pypath = configure.get_user_site_packages_path()
+def process_argv(argv: List[str]) -> Optional[bool]:
+    """
+    Check if ``argv`` should be processed.
+    """
+    pypath = configure.get_user_base_path()
+    assert pypath
     debug_print("process_argv", argv, pypath=pypath, pypath_exists=pypath.exists())
     if "TIAMAT_PIP_INSTALL" not in os.environ:
         log.debug("Not processing argv since TIAMAT_PIP_INSTALL is not in os.environ")
@@ -57,7 +74,7 @@ def process_argv(argv: List[str]) -> bool:
     cpath = os.environ.get("CPATH") or None
     c_include_path = os.environ.get("C_INCLUDE_PATH") or None
 
-    pyinstaller_extract_path = sys._MEIPASS  # pylint: disable=no-member
+    pyinstaller_extract_path = sys._MEIPASS  # type: ignore[attr-defined]
     included_python_headers_path = str(
         pathlib.Path(pyinstaller_extract_path).resolve() / "include" / "python"
     )
@@ -80,12 +97,12 @@ def process_argv(argv: List[str]) -> bool:
             # Example:
             #   python -c "print 'Foo!'"
             run_code(argv[2:])
-            return
+            return None
         elif argv[1] == "-u" and argv[2] == "-c":
             # Example:
             #   python -u -c "print 'Foo!'"
             run_code(argv[3:])
-            return
+            return None
 
         try:
             argv1_file = pathlib.Path(argv[1]).resolve()
@@ -93,7 +110,7 @@ def process_argv(argv: List[str]) -> bool:
                 # Example:
                 #   python this-is-a-script.py arg1 arg2
                 run_python_file(argv[1:])
-                return
+                return None
         except ValueError:
             # Not a valid file
             pass
@@ -107,11 +124,15 @@ def process_argv(argv: List[str]) -> bool:
 
 
 def process_pip_argv(argv: List[str]) -> None:
-    pypath = configure.get_user_site_packages_path()
+    """
+    Process pip ``argv``.
+    """
+    pypath = configure.get_user_base_path()
+    assert pypath
     debug_print("process_pip_argv", argv, pypath=pypath, pypath_exists=pypath.exists())
     if pypath is None:
         raise RuntimeError(
-            "You need to run 'tiamatpip.configure.set_user_site_packages_path(<path>)' "
+            "You need to run 'tiamatpip.configure.set_user_base_path(<path>)' "
             "before calling tiamatpip.cli.process_pip_argv()"
         )
 
@@ -128,11 +149,15 @@ def process_pip_argv(argv: List[str]) -> None:
         "TIAMAT_PIP_PYPATH": str(pypath),
     }
     with patched_environ(environ=environ):
-        return process_argv(argv)
+        process_argv(argv)
 
 
 def redirect_to_pip(argv: List[str]) -> None:
-    pypath = configure.get_user_site_packages_path()
+    """
+    Redirect ``argv`` to pip.
+    """
+    pypath = configure.get_user_base_path()
+    assert pypath
     debug_print("redirect_to_pip", argv, pypath=pypath, pypath_exists=pypath.exists())
     targets: Sequence[str] = ("install", "list", "freeze", "uninstall")
     try:
@@ -144,15 +169,12 @@ def redirect_to_pip(argv: List[str]) -> None:
         print(msg, file=sys.stderr, flush=True)
         sys.exit(1)
 
-    user_site_path = configure.get_user_site_packages_path()
     extra_environ = {
         "PIP_DISABLE_PIP_VERSION_CHECK": "1",
-        # The environment variable PYTHONUSERBASE can also be used to
-        # tell pip where to install packages into
-        "PYTHONUSERBASE": str(user_site_path),
     }
 
     # Valid command found
+    user_site_path = configure.get_user_site_path()
 
     if cmd in ("install", "uninstall"):
         include_in_store = True
@@ -175,7 +197,7 @@ def redirect_to_pip(argv: List[str]) -> None:
             args.extend(["--target", str(user_site_path)])
     elif cmd == "uninstall":
         args = [cmd]
-        patch_pip_get_installed_distributions()
+        patch_pip_internal_metadata_get_default_environment()
         extra_environ["TIAMAT_PIP_UNINSTALL"] = "1"
     elif cmd in ("list", "freeze"):
         args = [cmd, "--path", str(user_site_path)]
@@ -183,6 +205,7 @@ def redirect_to_pip(argv: List[str]) -> None:
         args = [cmd]
     args.extend(argv[3:])
     pkgs = []
+    parser: Union[InstallCommand, UninstallCommand]
     if cmd in ("install", "uninstall"):
         if cmd == "install":
             parser = InstallCommand("name", "summary")
@@ -210,7 +233,8 @@ def redirect_to_pip(argv: List[str]) -> None:
                 store.write()
             sys.exit(exitcode)
         finally:
-            pypath = configure.get_user_site_packages_path()
+            pypath = configure.get_user_base_path()
+            assert pypath
             debug_print(
                 "redirect_to_pip finally",
                 args,
@@ -219,12 +243,19 @@ def redirect_to_pip(argv: List[str]) -> None:
             )
 
 
-def call_pip(argv):
+def call_pip(argv: List[str]) -> int:
+    """
+    Call ``pip``.
+    """
     debug_print("call_pip", argv)
-    return pip_main(argv)
+    exitcode: int = pip_main(argv)
+    return exitcode
 
 
-def run_code(argv):
+def run_code(argv: List[str]) -> None:
+    """
+    Run a code string.
+    """
     source, *_ = argv
     with patched_sys_argv(argv):
         debug_print("run_code", argv)
@@ -232,7 +263,10 @@ def run_code(argv):
         interpreter.runsource(source)
 
 
-def run_python_file(argv):
+def run_python_file(argv: List[str]) -> None:
+    """
+    Run a python file.
+    """
     debug_print("run_python_file", argv)
     python_file, *_ = argv
     with open(python_file) as rfh:
